
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utils_phy.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif


/***************************************
 * Data
 ***************************************/


/***************************************
 *** Code                            ***
 ***************************************/

/***************************************
 * WLAN Phy Table
 ***************************************/

/*
 * used by h4aWlanPhyTable. Fetch WlanPhy data for
 * phy<x> (where x=index) and populates the WlanPhy_t structure.
 */
int get_phy_info(int index, WlanPhy_t *wphy){
    int len;
    char buffer[50];
    char *needle, *s;
    FILE *f;
    int i;

    if (wphy == NULL) return STS_FAILURE;

    /* initialize wphy structure */
    memset (wphy, 0, sizeof(WlanPhy_t));

    len=snprintf(wphy->h4aWlanPhyName, LEN_WLANPHYNAME, 
                 "phy%i", index);

    wphy->h4aWlanPhyName_len=MIN(len, LEN_WLANPHYNAME);

    /* prepare path for phyx/info */
    len=sprintf(buffer, "/proc/sys/dev/phy%i/info", index);
#ifdef _TEST
    len=sprintf(buffer, "./sample_input/info.in");
#endif
    /* open file */
    f=fopen(buffer, "r");

    if (!f) return STS_OPENERROR;

    /* read line by line,
     * strstr() for key phrases and sscanf the values from line buffer */
    while (f && fgets(buffer, sizeof(buffer), f)) {
#ifdef _TEST
        printf("line: %s\n",buffer);
#endif
        /* skip empty lines */
        if (buffer[0] == '\n') continue;

        needle="TX Pwr Lvl:";
        s=strstr(buffer, needle);
        if (s) {
            s = s + strlen(needle);
            len = sscanf(s, "%i", &(wphy->h4aWlanPhyTxPower));
        }

        needle="Noisefloor:";
        s=strstr(buffer, needle);
        if (s) {
            s = s + strlen(needle);
            len = sscanf(s, "%i", &(wphy->h4aWlanPhyNoisefloor));
        }

        needle="Chan BW:";
        s=strstr(buffer, needle);
        if (s) {
            s = s + strlen(needle);
            len = sscanf(s, "%i", &(wphy->h4aWlanPhyBandwidth));
        }
    } // while

    /* close file */
    if (f) { i=fclose(f);}

    /* prepare filename to /sys/class.../phyx file */
 
    len=sprintf(buffer, "/sys/class/ieee80211/phy%i/macaddress", index);
#ifdef _TEST
    len=sprintf(buffer, "./sample_input/mac.in");
#endif

    i = get_mac_from_file(buffer, wphy->h4aWlanPhyMac);

    /* return with success */
    return STS_SUCCESS;
}

