
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utils_stn.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif


/***************************************
 * Data
 ***************************************/
/* List containing all data records for WlanStn */
static QHead_t WlanStnHEAD={NULL};			// List head
WlanStn_t      *pWlanStnHEAD=(WlanStn_t*)&WlanStnHEAD;	// pointer to list


/***************************************
 *** Code                            ***
 ***************************************/

/***************************************
 * WLAN Station Table
 ***************************************/

/*
 * populate the WlanStn queue
 */
int update_stn_records(void) {
    int sts;
    int i;
    int len;
    char buffer[64];
    char *b;
    FILE *f;
    WlanStn_t stn;
    char *n;
    int wlanidx;

    // loop through the list and mark all entries as inactive.
    invalidate_stn_records();

    // get the WLAN station data
#ifdef _TEST
    f = popen ("cat ./sample_input/iw_wlan", "r");
#else
    f = popen ("for each in `iw dev|grep Interface|cut -f2 -d' '` ;do "
               "iw dev $each station dump;"
               "done", "r");
#endif
    if (f == NULL) return STS_FAILURE;

    memset (&stn, 0, sizeof(stn));

    while (! feof(f)) {
        b = fgets(buffer, sizeof(buffer)-1, f);
        if (b == NULL) break;
        if (strlen(b) <=0) continue;
        if (b[strlen(b)-1] == '\n') {b[strlen(b)-1]='\0';}
#ifdef _TEST
        printf("**%s**\n",b);
#endif
        /* process the line */

        #define N_STATION "Station "
        if ((n=strstr(b, N_STATION)) != NULL) {
            /* Station MAC remote & local WLAN IF */
            unsigned int vu[LEN_WLANSTNMAC];
            n+=strlen(N_STATION);


            len = sscanf(n, "%x:%x:%x:%x:%x:%x (on %s)", 
                         &vu[0], &vu[1], &vu[2], &vu[3], &vu[4], &vu[5],
                         stn.h4aWlanStnIfName);

            if (len >= LEN_WLANSTNMAC) {
                for (i=0; i<LEN_WLANSTNMAC; i++) {
                    stn.h4aWlanStnMacRemote[i]=(char)vu[i];
                } /* for */
            } /* if */

            if (len >= LEN_WLANSTNMAC+1) {
                /* also got the WLAN interface name */
                /* strip off trailing ')' character */
                if ((n=strchr(stn.h4aWlanStnIfName, ')')) != NULL) *n = '\0';
            } /* if */

            /* get local MAC for interface stn.h4aWlanStnIfName */
            sprintf(buffer, "/sys/class/net/%s/address", stn.h4aWlanStnIfName);
            sts = get_mac_from_file(buffer, stn.h4aWlanStnMacLocal);

 
#ifdef _TEST
printf("C:Station - len=%i, macr=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x if=%s\n",len,
       stn.h4aWlanStnMacRemote[0]&0xff, stn.h4aWlanStnMacRemote[1]&0xff,
       stn.h4aWlanStnMacRemote[2]&0xff, stn.h4aWlanStnMacRemote[3]&0xff,
       stn.h4aWlanStnMacRemote[4]&0xff, stn.h4aWlanStnMacRemote[5]&0xff,
       stn.h4aWlanStnIfName);
printf("C:Station - len=%i, macl=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x if=%s\n",len,
       stn.h4aWlanStnMacLocal[0]&0xff, stn.h4aWlanStnMacLocal[1]&0xff,
       stn.h4aWlanStnMacLocal[2]&0xff, stn.h4aWlanStnMacLocal[3]&0xff,
       stn.h4aWlanStnMacLocal[4]&0xff, stn.h4aWlanStnMacLocal[5]&0xff,
       stn.h4aWlanStnIfName);
#endif

        } else if ((n=strstr(b, "inactive time:")) != NULL) {
            /* inactive time */
        } else if ((n=strstr(b, "rx bytes:")) != NULL) {
            /* rx bytes */

        #define N_SIGNAL "signal:"
        } else if ((n=strstr(b, N_SIGNAL)) != NULL) {
            /* signal */
            int vi;
            n+=strlen(N_SIGNAL);
            len = sscanf(n, " %i", &vi);
            if (len > 0) {
                stn.h4aWlanStnRxLevel = vi;
            }
#ifdef _TEST
printf("C:signal - len=%i, v=%i N=[%s]\n",len, stn.h4aWlanStnRxLevel, n);
#endif
        #define N_RX_RATE "tx bitrate:"
        } else if ((n=strstr(b, N_RX_RATE)) != NULL) {
            /* TX bitrste */
            float vf=0;
            n+=strlen(N_RX_RATE);
            len = sscanf(n, " %f", &vf);
            if (len > 0) {
                stn.h4aWlanStnTxRate = vf * 1000*1000;	/* f is in MBit/s */
            }
#ifdef _TEST
printf("C:rxrate - len=%i, v=%i n=[%s]\n",len, stn.h4aWlanStnTxRate, n);
printf("EOR\n");
#endif

// figure out wlan index
wlanidx=0;
for (i=0;i<strlen(stn.h4aWlanStnIfName); i++) {
    // skip non numeric digits
    if ((stn.h4aWlanStnIfName[i] < '0') || (stn.h4aWlanStnIfName[i] > '9')) continue;
    wlanidx=atoi(&stn.h4aWlanStnIfName[i]);
    break;
}
            sts = insert_stn_record(wlanidx, &stn);
            memset (&stn, 0, sizeof(stn));
        } /* if */

    } /* while */
    sts = pclose(f);

    return STS_SUCCESS;
}


/*
 * get next WlanStn record from list
 */
WlanStn_t* get_next_stn_record(WlanStn_t *ptr) {
    if (ptr == NULL) ptr = pWlanStnHEAD;
    return queue_get_next(ptr);
}


/*
 * invalidate all Stn records in lists.
 */
void invalidate_stn_records(void) {
    WlanStn_t *ptr=NULL;

    while ((ptr=get_next_stn_record(ptr)) != NULL) {
        ptr->active=0;
    }
    return;
}


/*
 * Insert a WlanStn_t record
 * update an existing record or insert a new record
 * Indexes are assigned as follows:
 * wlan x 1000 + index
 *	wlan0: 1, 2, 3, ...
 *	wlan1: 1001, 1002, 1003, ...
 * Key is Remote MAC address
 * new records are inserted, old records are kept in the list
 * to occupy the index value.
 */
int insert_stn_record(int wlanidx, WlanStn_t *stn) {
    WlanStn_t *ptr=NULL;
    WlanStn_t *ptr_insert_after=NULL;

    if (stn == NULL) return STS_FAILURE;
    if (wlanidx < 0) return STS_FAILURE;

    /* loop through the list */
    for (ptr = pWlanStnHEAD->next; ptr != NULL; ptr=ptr->next) {
        if (ptr == NULL) break;

        /* break from loop if too high wlan<x> (means: insert before this record) */
        if (ptr->h4aWlanStnIndex  > ((wlanidx+1) * RANGE_WLANSTFINDEX)) {
            ptr = NULL;
            break;
        }

        /* skip entries with too low wlan<x> indices */
        if (ptr->h4aWlanStnIndex  <= (wlanidx * RANGE_WLANSTFINDEX)) {
            ptr_insert_after = ptr;
            continue;
        }

        /* Here, only records from the same wlan index range are processed. */

        /* remember last list entry */
        ptr_insert_after = ptr;
        
        /* Match of RemoteMAC? -> UPDATE existing entry */
        if (memcmp(stn->h4aWlanStnMacRemote, ptr->h4aWlanStnMacRemote, LEN_WLANSTNMAC) == 0) {
            // UPDATE existing record
            #if TEST
            printf("**UPDATE %p\n",ptr);
            #endif
            //memcpy(&(ptr->active), &(stn->active), sizeof(WlanStn_t)-4);
            //   h4aWlanStnIndex;			// does not change
            strcpy(ptr->h4aWlanStnIfName, stn->h4aWlanStnIfName);
            //   h4aWlanStnMacLocal[LEN_WLANSTNMAC];	// does not change
            //   h4aWlanStnMacRemote[LEN_WLANSTNMAC];	// does not change
            ptr->h4aWlanStnRxLevel = stn->h4aWlanStnRxLevel;
            ptr->h4aWlanStnTxRate  = stn->h4aWlanStnTxRate;

            ptr->active=1;

            // no insert needed
            break;
        }
    } // for

    /* ptr == NULL: reached end-of list without having processed the entry
     * ptr != NULL: updated the entry *ptr */
    if (ptr == NULL) {
        /* need to insert a new element */
        WlanStn_t *wrk=malloc(sizeof(WlanStn_t));

        /* allocate memory for new list entry */
        if (wrk == NULL) return STS_FAILURE;

        #if TEST
        printf("**INSERT %p\n",ptr_insert_after);
        #endif

        /* clone */
        memcpy(wrk, stn, sizeof(WlanStn_t));
        wrk->active=1;

        /* get next index value for MIB */
        if ((ptr_insert_after == NULL) ||
            (ptr_insert_after->h4aWlanStnIndex <= (wlanidx * RANGE_WLANSTFINDEX))) {
            wrk->h4aWlanStnIndex=(wlanidx * RANGE_WLANSTFINDEX)+1;
        } else {
                wrk->h4aWlanStnIndex=ptr_insert_after->h4aWlanStnIndex+1;
        }

        /* insert new element to list */
        if (ptr_insert_after == NULL) {
            /* insert on queue head? (top of queue) */
            queue_insert(pWlanStnHEAD, wrk);
        } else {
            queue_insert(ptr_insert_after, wrk);
        }

    } // if ptr

    return STS_SUCCESS;
}
